package com.sun.corba.se.impl.ior;

import com.sun.corba.se.spi.ior.iiop.IIOPProfile;
import java.util.HashMap;
import com.sun.corba.se.spi.ior.TaggedProfile;
import org.omg.IOP.IORHelper;
import com.sun.corba.se.impl.encoding.EncapsOutputStream;
import java.io.IOException;
import com.sun.corba.se.impl.orbutil.HexOutputStream;
import java.io.StringWriter;
import sun.corba.OutputStreamFactory;
import org.omg.CORBA_2_3.portable.OutputStream;
import org.omg.CORBA_2_3.portable.InputStream;
import java.util.Iterator;
import com.sun.corba.se.spi.ior.ObjectKeyTemplate;
import com.sun.corba.se.spi.ior.TaggedProfileTemplate;
import com.sun.corba.se.spi.ior.IORFactories;
import com.sun.corba.se.spi.ior.ObjectId;
import com.sun.corba.se.spi.ior.IORTemplate;
import com.sun.corba.se.spi.ior.IORTemplateList;
import com.sun.corba.se.impl.logging.IORSystemException;
import com.sun.corba.se.spi.orb.ORB;
import com.sun.corba.se.spi.ior.IOR;
import com.sun.corba.se.spi.ior.IdentifiableContainerBase;

public class IORImpl extends IdentifiableContainerBase implements IOR
{
    private String typeId;
    private ORB factory;
    private boolean isCachedHashValue;
    private int cachedHashValue;
    IORSystemException wrapper;
    private IORTemplateList iortemps;
    
    public ORB getORB() {
        return this.factory;
    }
    
    public boolean equals(Object o) {
        if (o == null) {
            return false;
        }
        if (!(o instanceof IOR)) {
            return false;
        }
        IOR ior = (IOR)o;
        return super.equals(o) && this.typeId.equals(ior.getTypeId());
    }
    
    public synchronized int hashCode() {
        if (!this.isCachedHashValue) {
            this.cachedHashValue = (super.hashCode() ^ this.typeId.hashCode());
            this.isCachedHashValue = true;
        }
        return this.cachedHashValue;
    }
    
    public IORImpl(ORB orb) {
        this(orb, "");
    }
    
    public IORImpl(ORB factory, String typeId) {
        this.factory = null;
        this.isCachedHashValue = false;
        this.iortemps = null;
        this.factory = factory;
        this.wrapper = IORSystemException.get(factory, "oa.ior");
        this.typeId = typeId;
    }
    
    public IORImpl(ORB orb, String s, IORTemplate iorTemplate, ObjectId objectId) {
        this(orb, s);
        (this.iortemps = IORFactories.makeIORTemplateList()).add(iorTemplate);
        this.addTaggedProfiles(iorTemplate, objectId);
        this.makeImmutable();
    }
    
    private void addTaggedProfiles(IORTemplate iorTemplate, ObjectId objectId) {
        ObjectKeyTemplate objectKeyTemplate = iorTemplate.getObjectKeyTemplate();
        Iterator<Object> iterator = iorTemplate.iterator();
        while (iterator.hasNext()) {
            this.add(((TaggedProfileTemplate)(TaggedProfileTemplate)iterator.next()).create(objectKeyTemplate, objectId));
        }
    }
    
    public IORImpl(ORB orb, String s, IORTemplateList iortemps, ObjectId objectId) {
        this(orb, s);
        this.iortemps = iortemps;
        Iterator<Object> iterator = iortemps.iterator();
        while (iterator.hasNext()) {
            this.addTaggedProfiles((IORTemplate)(IORTemplate)iterator.next(), objectId);
        }
        this.makeImmutable();
    }
    
    public IORImpl(InputStream inputStream) {
        this((ORB)(ORB)inputStream.orb(), inputStream.read_string());
        EncapsulationUtility.readIdentifiableSequence(this, this.factory.getTaggedProfileFactoryFinder(), inputStream);
        this.makeImmutable();
    }
    
    public String getTypeId() {
        return this.typeId;
    }
    
    public void write(OutputStream outputStream) {
        outputStream.write_string(this.typeId);
        EncapsulationUtility.writeIdentifiableSequence(this, outputStream);
    }
    
    public String stringify() {
        EncapsOutputStream encapsOutputStream = OutputStreamFactory.newEncapsOutputStream(this.factory);
        encapsOutputStream.putEndian();
        this.write((OutputStream)encapsOutputStream);
        StringWriter obj = new StringWriter();
        try {
            encapsOutputStream.writeTo(new HexOutputStream(obj));
        }
        catch (IOException ex) {
            throw this.wrapper.stringifyWriteError(ex);
        }
        return "IOR:" + obj;
    }
    
    public synchronized void makeImmutable() {
        this.makeElementsImmutable();
        if (this.iortemps != null) {
            this.iortemps.makeImmutable();
        }
        super.makeImmutable();
    }
    
    public org.omg.IOP.IOR getIOPIOR() {
        EncapsOutputStream encapsOutputStream = OutputStreamFactory.newEncapsOutputStream(this.factory);
        this.write(encapsOutputStream);
        return IORHelper.read((InputStream)(InputStream)encapsOutputStream.create_input_stream());
    }
    
    public boolean isNil() {
        return this.size() == 0;
    }
    
    public boolean isEquivalent(IOR ior) {
        Iterator<Object> iterator = this.iterator();
        Iterator<Object> iterator2 = ior.iterator();
        while (iterator.hasNext() && iterator2.hasNext()) {
            if (!((TaggedProfile)(TaggedProfile)iterator.next()).isEquivalent((TaggedProfile)(TaggedProfile)iterator2.next())) {
                return false;
            }
            continue;
        }
        return iterator.hasNext() == iterator2.hasNext();
    }
    
    private void initializeIORTemplateList() {
        HashMap<ObjectKeyTemplate, IORTemplate> hashMap = new HashMap();
        this.iortemps = IORFactories.makeIORTemplateList();
        Iterator<Object> iterator = this.iterator();
        ObjectId objectId = null;
        while (iterator.hasNext()) {
            TaggedProfile taggedProfile = (TaggedProfile)(TaggedProfile)iterator.next();
            TaggedProfileTemplate taggedProfileTemplate = taggedProfile.getTaggedProfileTemplate();
            ObjectKeyTemplate objectKeyTemplate = taggedProfile.getObjectKeyTemplate();
            if (objectId == null) {
                objectId = taggedProfile.getObjectId();
            }
            else if (!objectId.equals(taggedProfile.getObjectId())) {
                throw this.wrapper.badOidInIorTemplateList();
            }
            IORTemplate iorTemplate = (IORTemplate)(IORTemplate)hashMap.get(objectKeyTemplate);
            if (iorTemplate == null) {
                iorTemplate = IORFactories.makeIORTemplate(objectKeyTemplate);
                hashMap.put(objectKeyTemplate, iorTemplate);
                this.iortemps.add(iorTemplate);
            }
            iorTemplate.add(taggedProfileTemplate);
        }
        this.iortemps.makeImmutable();
    }
    
    public synchronized IORTemplateList getIORTemplates() {
        if (this.iortemps == null) {
            this.initializeIORTemplateList();
        }
        return this.iortemps;
    }
    
    public IIOPProfile getProfile() {
        IIOPProfile iiopProfile = null;
        Iterator<Object> iteratorById = this.iteratorById(0);
        if (iteratorById.hasNext()) {
            iiopProfile = (IIOPProfile)(IIOPProfile)iteratorById.next();
        }
        if (iiopProfile != null) {
            return iiopProfile;
        }
        throw this.wrapper.iorMustHaveIiopProfile();
    }
}
